#ifndef BOOTSTRAP_BOOTSTRAP_H
#define BOOTSTRAP_BOOTSTRAP_H


#include <vector>
#include <random>
#include <iostream>
#include <json/json.h>
#include "device.h"
#include "devicemanager.h"
#include <flags.h>


class Bootstrap {
public:
    Bootstrap(std::string iter_counts_files);
    virtual ~Bootstrap() {};

    std::vector<float> bootstrap(std::vector<float> input, int n_iterations, float p, Flags flags = Flags::NONE,
    float manual_gpu_fraction = -1.0f);

    Timer total_time;

private:
    void prepareInput(std::vector<float> input);

    std::vector<int> getLoadBalance(std::vector<float> input, int n_iterations, float p);

    // Element at index #n = sum(input[:n]) (Python notation)
    std::vector<float> cumulativeInput;

    std::vector<std::unique_ptr<DeviceManager>> device_managers;
    std::vector<std::unique_ptr<Device>> devices;

    std::string iter_counts_file;
};

class BootstrapException : Exception {
public:
    BootstrapException(std::string message) : Exception(message) {};
};


#endif //BOOTSTRAP_BOOTSTRAP_H
