#ifndef BOOTSTRAP_CPUDEVICE_H
#define BOOTSTRAP_CPUDEVICE_H

#include "device.h"
#include "estimator.h"

class CpuDevice : public Device {
public:
    CpuDevice(int thread_id);
    ~CpuDevice();

    virtual std::string getDeviceName() const override;

    virtual std::string getDeviceType() const override {
        return std::string("CPU");
    }

    std::vector<float> run(std::vector<float> cumulative_input,
                           int n_iterations,
                           float p,
                           Flags flags);


    double getCost(int n_iterations) override;
    double getDCost(int n_iterations) override;

    void train(int n_iterations, double cost);

    std::shared_ptr<Estimator> estimator_ptr;
private:

    int thread_id;
    std::mt19937 randomGenerator;
    void initializeRandomGenerator();
};

#endif //BOOTSTRAP_CPUDEVICE_H
