#ifndef BOOTSTRAP_DESERIALIZER_H
#define BOOTSTRAP_DESERIALIZER_H


#include <fstream>
#include <vector>
#include "utils.h"

class Deserializer {
public:
    Deserializer(std::string filename) : read_pos(0)
    {
        std::ifstream input(filename, std::ios::in | std::ios::binary | std::ios::ate);

        int file_size_bytes = (int) input.tellg();

        input.seekg(0, std::ios::beg);

        stream.resize((unsigned long) file_size_bytes);

        input.read((char *) stream.data(), file_size_bytes);

        input.close();
    }

    template <typename T>
    void deserialize(T &obj);

private:
    std::vector<uint8_t> stream;
    int read_pos;
};

template<typename T>
void Deserializer::deserialize(T &obj) {
    int obj_size = sizeof (T);

    if (read_pos + obj_size > (int) stream.size()) {
        throw Exception("No more data to deserialize!");
    }

    uint8_t *data_ptr = stream.data() + read_pos;

    T *temp = reinterpret_cast<T *>(data_ptr);

    obj = *temp;

    read_pos += obj_size;
}

#endif //BOOTSTRAP_DESERIALIZER_H
