#ifndef BOOTSTRAP_ESTIMATOR_H
#define BOOTSTRAP_ESTIMATOR_H

#include "datatable.h"
#include "bspline.h"
#include "bsplinebuilder.h"


class Estimator {
public:
    Estimator(std::string savePath);
    virtual ~Estimator();

    bool canEstimate();

    void train(int n_iterations, double cost);
    void train(int input_size, int n_iterations, double cost);
    void train(int input_size, int n_iterations, float p, double cost);

    void save();

    double estimate(int n_iterations);
    double estimate(int input_size, int n_iterations);
    double estimate(int input_size, int n_iterations, float p);

    double dEstimate(int n_iterations);
    double dEstimate(int input_size, int n_iterations);
    double dEstimate(int input_size, int n_iterations, float p);

    void setAlpha(double newAlpha);

    void rebuildIfNecessary();

    std::string getDataTablePath();
    std::string getBSplinePath();

private:
    bool need_rebuild;
    std::string basePath;
    double alpha;

    std::unique_ptr<SPLINTER::BSpline> bSpline_ptr;
    std::unique_ptr<SPLINTER::DataTable> dataTable_ptr;
};

#endif //BOOTSTRAP_ESTIMATOR_H
