#ifndef BOOTSTRAP_FLAGS_H
#define BOOTSTRAP_FLAGS_H


#include <type_traits>
#include <ostream>

struct Flags {
    enum {
        NONE                = 0x0,
        SORT_CPU            = 0x1,
        SORT_GPU            = 0x2,
        GROUP_INTO_WARPS    = 0x4,
        THRESHOLD           = 0x8,
        MANUAL_LOAD_BALANCE = 0x10,
        GENERATE_ITER_CTS   = 0x20,
        GENERATE_SEEDS_CPU  = 0x40,
        GENERATE_SEEDS_GPU  = 0x80,
    } Values;

    Flags() {
        Values = Flags::NONE;
    }

    Flags(std::underlying_type<decltype(Values)>::type value) {
        Values = static_cast<decltype(Values)>(value);
    }

    explicit operator bool() const;
};

bool operator!(const Flags& flags);
Flags operator&(const Flags& lhs, int rhs);
Flags operator&(int lhs, const Flags& rhs);
Flags operator|(const Flags& lhs, int rhs);
Flags operator|(int lhs, const Flags &rhs);
Flags &operator|=(Flags& lhs, int rhs);

std::ostream& operator<<(std::ostream& out, const Flags& flags);

#endif //BOOTSTRAP_FLAGS_H
