#ifndef BOOTSTRAP_OPERATION_H
#define BOOTSTRAP_OPERATION_H

#include <functional>
#include <estimator.h>
#include <memory>
#include "utils.h"


class CudaDevice;

class Operation {
    typedef void (CudaDevice::*memberFunc)(void);
public:
    Operation(const std::string &operationName);
    Operation(const std::string &deviceName, const std::string id, const std::string &operationName, memberFunc opFunc,
              CudaDevice *self, bool inputSizeDependent=false, bool pDependent=false);

    Operation(const Operation &other);

    virtual ~Operation();

    unsigned int id;

    double cost(int input_size, int n_iterations, float p);
    // Derivative of the cost
    double dCost(int input_size, int n_iterations, float p);

    std::string operationName;

    Operation &operator=(const Operation &other);

    void operator()(int input_size, int n_iterations, float p);

    void saveEstimator();

private:
    bool dummy;

    Timer operationTimer;

    memberFunc operation;
    CudaDevice *self;

    std::shared_ptr<Estimator> estimator_ptr;

    bool forceLowEstimate;
    bool inputSizeDependent;
    bool pDependent;

    static unsigned int next_id;
};

#endif //BOOTSTRAP_OPERATION_H
